import org.virtualbox_4_3.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;

public class Main {
    final static String source = "Ubuntu 1";
    final static String cloneName = "Ubuntu 1 Clone 1";

    static void clone(IVirtualBox vbox) {
        IMachine targetMachine = vbox.createMachine("", cloneName, new ArrayList<String>(), "Ubuntu_64", "");
        IMachine sourceMachine = vbox.findMachine(source);

        IProgress progress = sourceMachine.getCurrentSnapshot().getMachine().cloneTo(targetMachine, CloneMode.MachineState, Arrays.asList(CloneOptions.Link));

        while (!progress.getCompleted() && !progress.getCanceled()) {
            progress.waitForCompletion(500);
            System.out.println("Cloning: Remaining time is: " + progress.getTimeRemaining());
        }
        System.out.println("Cloning: Completed");

        targetMachine.setGroups(Arrays.asList("/Clones")); // THIS CAUSES THE PROBLEM in the end

        targetMachine.saveSettings();

        vbox.registerMachine(targetMachine);

        // VERIFY: Files created in 'PATH_TO_VM_FILES/Clones/Ubuntu 1 Clone 1'
        // instead of in 'PATH_TO_VM_FILES/Ubuntu 1 Clone 1' when not using a group
    }

    static void delete(IVirtualBox vbox) {


        IMachine machine = vbox.findMachine(cloneName);
        List<IMedium> liMedium = machine.unregister(CleanupMode.DetachAllReturnHardDisksOnly);

        // WORKAROUND: A possible workaround (make sure you start with a proper configuration of VB)
//        machine.setGroups(Arrays.asList("/"));
//        machine.saveSettings();

        IProgress progress = machine.deleteConfig(liMedium);
        while (!progress.getCompleted() && !progress.getCanceled()) {
            progress.waitForCompletion(500);
            System.out.println("Deleting: Remaining time is: " + progress.getTimeRemaining());
        }
        System.out.println("Deleting: Completed");

        // VERIFY: Files and folder are not deleted. 'PATH_TO_VM_FILES/Clones/Ubuntu 1 Clone 1' still exists
        // BUG! On second execution of this program, execution will fail due to that
    }


    public static void main(String[] args) {
        VirtualBoxManager mgr = VirtualBoxManager.createInstance(null);

        // WARNING> We assume here that the authentication was manually disabled in VirtualBox using
        // > VBoxManager.exe VBoxManage setproperty websrvauthlibrary null
        String url = "http://localhost:18083/";
        String user = "test";
        String passwd = "test";
        try {
            System.out.println("Connecting to " + url + " as " + user + " with " + passwd);
            mgr.connect(url, user, passwd);
            System.out.println("Connected");
        } catch (VBoxException e) {
            e.printStackTrace();
            System.out.println("Cannot connect, start webserver first!");
        }

        try {
            IVirtualBox vbox = mgr.getVBox();
            if (vbox != null) {
                System.out.println("VirtualBox version: " + vbox.getVersion() + "\n");

                clone(vbox);
                delete(vbox);

                System.out.println("done, press Enter...");
                int ch = System.in.read();
            }
        } catch (VBoxException e) {
            System.out.println("Java stack trace:");
            e.printStackTrace();
        } catch (RuntimeException e) {
            System.out.println("Runtime error: " + e.getMessage());
            e.printStackTrace();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }

        try {
            mgr.disconnect();
        } catch (VBoxException e) {
            e.printStackTrace();
        }

        mgr.cleanup();

    }

}
